//biblioteca responsável pela comunicação com o display LCD
#include <SoftwareSerial.h>
//biblioteca responsável por capturar a tecla que foi pressionada no teclado
#include <Keypad.h>

// pino invalido apenas usado no contrutor do SoftwareSerial
#define RX_PIN   255   
// pino TX da nossa ligação Arduino x Display LCD   
#define TX_PIN   3 
// inverte a lógica dos pinos Rx e Tx, tratando LOW como HIGH e vice-versa       
#define INVERTED 1 

/*
  TODOS OS COMANDOS DEVEM VIR PRECEDIDO DO PREFIXO: 254
  EX: 254,<comando>
*/
//comando para limpar toda a tela do display
const char limparTela[ ] = {
  254,1};

//comandos de configuração inicial
/*
  254,254 --> acende o backlight
  254,1   --> limpa a tela
  254,253,1 --> configura o contraste em nível alto
  254,13   --> liga o cursor paraficar piscando
*/
const char configInicial[ ] = {
  254,254, 254,1, 254,253,1, 254,13};

const byte LINHAS = 4; //número de linhas do teclado
const byte COLUNAS = 4; //número de colunas do teclado

//define uma matriz com os símbolos que deseja ser lido do teclado
char SIMBOLOS[LINHAS][COLUNAS] = {
  {'A','1','2','3'},
  {'B','4','5','6'},
  {'C','7','8','9'},
  {'D','c','0','e'}
};

byte PINOS_LINHA[LINHAS] = {8, 9, 10, 11}; //pinos que indicam as linhas do teclado
byte PINOS_COLUNA[COLUNAS] = {4, 5, 6, 7}; //pinos que indicam as colunas do teclado

//instancia de Keypad, responsável por capturar a tecla pressionada
Keypad customKeypad = Keypad( makeKeymap(SIMBOLOS), PINOS_LINHA, PINOS_COLUNA, LINHAS, COLUNAS); 

//variáveis resposnsáveis por armazenar as senhas
const String SENHA_ESPERADA = "1234ABCD";
String SENHA_DIGITADA = "";

//instancia de SoftwareSerial para nos comunicar com o Display via serial
SoftwareSerial displaySerial =  SoftwareSerial(RX_PIN, TX_PIN, INVERTED);

void setup(){
  Serial.begin(2400);
  //inicializando a serial de comunicação com o display
  //importante o baud rate ser de 2400
  displaySerial.begin(2400);
  //tempo de espera pela inicialização do display
  delay(700);  
  //seta a configuração inicial do display
  displaySerial.print(configInicial);  
  delay(10);
  inicio();
}
  
void loop(){

  //captura a tecla pressionada do teclado
  char customKey = customKeypad.getKey();

  //caso alguma tecla foi pressionada
  if (customKey){
    Serial.println(customKey);
    switch(customKey)
    {
      //caso alguma das teclas imprimíveis foi pressionada
      case 'A':
      case 'B':
      case 'C':
      case 'D':
      case '0':
      case '1':
      case '2':
      case '3':
      case '4':
      case '5':
      case '6':
      case '7':
      case '8':
      case '9':
            //concatena o novo símbolo a senha que estamos digitando
            SENHA_DIGITADA+=customKey; 
            Serial.println(SENHA_DIGITADA);
            //imrpime na tela o símbolo pressionado
            displaySerial.print(customKey);
            break;
      //caso a tecla CLEAR tenha sido pressionada      
      case 'c':
              //limpa a variável que guarda a senha que está sendo digitada
              SENHA_DIGITADA = "";
              //chama o comando para limpar a tela
              displaySerial.print(limparTela);
              //configura a mensagem para digitar a senha
              inicio();
              break;
      //caso a tecla ENTER seja pressionada, devemos comparar as senhas
      case 'e':
              //limpa a tela
              displaySerial.print(limparTela);
              
              //se a senha digitada foi igual a ESPERADA              
              if(SENHA_ESPERADA==SENHA_DIGITADA)
              {
                Serial.println("Senha Correta!");
                //imprime mensagem de senha correta
                displaySerial.print("Senha Correta!!!");
              }
              //caso senha esteja errada
              else{
                Serial.println("Senha Incorreta!");
                //imprime mensagem de senha incorreta
                displaySerial.print("Senha Incorreta!");
              }
              //aguarda 2 segundos para limpar a tela novamente e esperar uma nova senha ser digitada
              delay(2000);
              displaySerial.print(limparTela);
              SENHA_DIGITADA = "";
              inicio();
              break;
      default: break;
      
    }
  }
}

//função responsável por imprimir na tela a mensagem para digitar a senha
//é chamada toda vez q a senha foi digitada e comparada, também quando 
//a tecla limpar display foi pressionada.
void inicio(){
  displaySerial.print("Senha:");
}

